'Programm ......... : Programmierbare Infrarot-Fernbedienung IRF
'
'Programminhalt ... : - Zusammenfassung von Sendebefehlen und deren bertragung
'                     - Dekodierung von IR-Befehlen und Darstellung auf einem
'                       LCD-Modul (Anschlu ber Steckverbinder)
'                     - automatische Abschaltung des MCs (Powerdown)
'
'Programmierer .... : Ingolf Bauer (ingolf.bauer@nexgo.de)
'Version .......... : 1.0
'Datum ............ : 01.08.2007
'
'-------------------------------------------------------------------------------
'
'Simulator: Einstellmglichkeit
'
'$sim                                             'Wartezeit aus
' Achtung!
' Unbedingt auskommentieren, bevor der AVR programmiert wird, da sonst der AVR
' nicht korrekt arbeitet.
'
'-------------------------------------------------------------------------------
'
'bersicht zur Software
' Inhalt                                          | Zeile
'--------------------------------------------------------
' Allgemeines                                     |
'    Abkrzungsbersicht ........................ |   60
'    Versionsbersicht .......................... |   70
'    Programmspeicher ........................... |   75
'    Compilereinstellungen ...................... |   79
'    Anschlu ................................... |   90
'    IR-Diode ................................... |   95
'    Frequenzgenerator .......................... |  100
'    Interrupt .................................. |  115
'    Einstellung der FUSE-Bits mit TwinAvr ...... |  121
'    Anschluschema fr den AVR ATmega16 ........ |  144
'    I/O-Ports .................................. |  170
'    Steckverbinder ............................. |  251
' Definition .................................... |  302
' Compilereinstellung ........................... |  307
' Deklaration ................................... |  315
' Konfiguration                                   |
'    Port ....................................... |  380
'    Timer ...................................... |  410
'    LCD ........................................ |  434
'    Interrupt .................................. |  445
' Programm                                        |
'    Initialisierung ............................ |  467
'    Programmanfang ............................. |  476
'    Unterprogramme ............................. |  519
'    Funktion ................................... |  930
'    Interruptbearbeitung ....................... |  960
' Datenbereich .................................. | 1089
'
'-------------------------------------------------------------------------------
'
'Allgemeine Informationen
'========================
'
'01. Abkrzungsbersicht
'-----------------------
' M  .......... : Minute(n)
' RC .......... : Remotecontrol
'                 - Taste der Fernbedienung, deren Code ermittelt werden soll
' Rx .......... : Empfangen
' S  .......... : Sekunde(n)
' SS .......... : serielle Schnittstelle
' Tx .......... . Senden
'
'02. Versionsbersicht
'---------------------
'V 1.0   01.08.2007 - Powerdown ... : - AVR schaltet in einen Ruhezustand
'V 1.0   01.05.2007 - Inhalt) ..... : - Erstellung
'
'03. Programmspeicher
'--------------------
' Programmspeicher des MCs (Programm-Flashspeicher): 41%
'
'04. Compilereinstellungen
'-------------------------
' Im Men (Compiler-Chip) vor dem Compilieren mindestens einstellen mit:
' > HW Stack 96   Soft Stack 48   Framesize 48
'
' Es kommt sonst zu undefinierten Sprngen, falschen Anzeigen u. . Dingen, wo-
' ran man beim Testen des Programms schnell verzweifeln kann.
' Auerdem mu auf die genaue Angabe des Chips (M16) geachtet werden.
' Andere MC der AVR-Serie knnen auch eingesetzt werden (z. B. der pinkompatible
' ATmega32).
'
'05. Anschlu
'------------
' Der Widerstand der Sendeeinheit (Dokumentation BASCOM) mu mit dem OC0-Pin
' des Mikrocontrollers verbunden werden. Beim ATmega16 ist dies Pin 4 (PB.3).
'
'06. IR-Diode
'------------
' Die IR-Diode wird ber einen PNP-Transisitor mit L-Signal eingeschaltet;
' PB.3= 1 bedeutet daher eine ausgeschaltete IR-Diode.
'
'07. Frequenzgenerator
'---------------------
' Register: TCCR0
' - Timer0 im CTC-Modus als Frequenzgenerator (36 kHz)
' - Prescale= 1, Voreinstellung im OCR0-Register (OCR0= 110)
' - freilaufend ohne zustzliche Interrupt-Routinen
' - Timer0-Stop mit TCCR0= &Bxxxx_x000
'
' Bit    7     6     5     4     3    2    1    0
'     FOC0 WGM00 COM01 COM00 WGM01 CS02 CS01 CS00
'        0     0     0     1     1    0    0    1
'                                     |----|----|--- Prescale ... : 1
'              |-----------------|------------------ Modus ...... : CTC
'                    |-----|------------------------ Modus ...... : Toggle
'
'08. Interrupt
'-------------
' ATMEL-Beschreibung Seite 66!
' Interrupt 2 mu als "Output" konfiguriert werden, damit ein Software-Interrupt
' den MC aus dem Powerdown-Modus erwachen lt.
'
'09. Einstellung der FUSE-Bits mit TwinAvr
'-----------------------------------------
' - Frequenzeinstellung (extern, Quarz)
'   CKSEL0 ........... : Haken ist gesetzt (Lieferzustand)
'   CKSEL1- CKSEL3 ... : Haken setzen (111) bei Quarzfrequenz 8,0 MHz

'   Achtung!
'   ========
'   Wenn aus Versehen CKSEL3=0 gesetzt wurde, arbeitet der MC nur noch mit einem
'   externen RC-Generator [f= 1/(3RC); C>= 22p].
'
' - JTAG(EN)
'   JTAGEN ........... : Haken setzen (Lieferzustand: Haken ist nicht gesetzt)
'
'   ACHTUNG!
'   ========
'   Beim ATmega16 wird das JTAG(EN) Flag defaultmig auf Enable gesetzt. Dies
'   bedeutet, da das JTAG-Interface (prozenahe Datenabfrage) aktiviert ist.
'   Dann kann aber Port C fr keine anderen Aufgaben genutzt werden!
'   Wie und ob das Flag programmig verndert werden kann, ist mir nicht be-
'   kannt. Auf alle Flle kann es ausgeschaltet werden, wenn unter "Configura-
'   tion" beim AVR-Programmer ein Haken bei JTAGEN gesetzt wird.
'
'10. Anschluschema fr den AVR ATmega16
'---------------------------------------
'
'                         --------------|_|--------------
'                        | 1 [PB0]               [PA0] 40| <--- T_0
'                        | 2 [PB1]               [PA1] 39| <--- T_1
'               WPM ---> | 3 [PB2, INT2]         [PA2] 38| <--- T_2
'               IRD <--- | 4 [PB3, OC0]          [PA3] 37| <--- T_3
'                        | 5 [PB4]               [PA4] 36| <--- T_4
'          PRG_MOSI ---> | 6 [PB5, MOSI]         [PA5] 35| <--- T_5
'          PRG_MISO <--- | 7 [PB6, MISO]         [PA6] 34| <--- T_6
'          PRG_SCK  ---> | 8 [PB7, SCK]          [PA7] 33| <--- T_7
'          PRG_R    ---> | 9 [/Reset]           [AREF] 32| ==== KON_A (analog)
'             P5V_D ==== |10 [VCC]              [AGND] 31| ==== GND_A (analog)
'             GND_D ==== |11 [GND]              [AVCC] 30| ==== P5V_A (analog)
'                QZ ++++ |12 [XTAL2]             [PC7] 29|
'                QZ ++++ |13 [XTAL1]             [PC6] 28|
'            RXD_KL ---> |14 [PD0, RXD]          [PC5] 27| ---> LCD_DB7_KL
'            TXD_KL <--- |15 [PD1, TXD]          [PC4] 26| ---> LCD_DB6_KL
'             IRE_0 ---> |16 [PD2, INT0]         [PC3] 25| ---> LCD_DB5_KL
'             IRE_1 ---> |17 [PD3, INT1]         [PC2] 24| ---> LCD_DB4_KL
'                        |18 [PD4]               [PC1] 23| ---> LCD_RS_KL
'             LED_1 <--- |19 [PD5]               [PC0] 22| ---> LCD_E_KL
'             LED_2 <--- |20 [PD6]               [PD7] 21|
'                         -------------------------------
'
'11. I/O-Ports
'-------------
' AE/ AA: analoger  Ein-/ Ausgang
' DE/ DA: digitaler Ein-/ Ausgang
'
' _A    : Analogteil
' _D    : Digitalteil
' _KL   : Anschlu nur auf Klemmenleiste gefhrt
' _(n)  : n: Nummer (Tastatur)
' _DB(d): d: Data Bit (Display)
' _RS   : Data/ Instruction (Display)
' _E    : Enable (Display)
'
' GND_  : Stromversorgung (Bezugspotential)
' IRE_  : dekodiertes Signal einer Fernbedienung
' KON   : Kondensator
' LCD   : LCD-Anzeige
' LED_  : LED-Steuerung
' PRG   : Programmierung (Signale MOSI, MISO, SCK, Reset)
' P5V_  : Stromversorgung +5V
' QZ    : Quarz
' SE    : Sende- und Empfnger
' WPM   : Wecken aus Powerdown-Modus
' SV    : Stromversorgung
' T     : Tastatur
'
'           Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' -   : SV  P5V_D ...... | ............. | Stromversorgung Digitalteil .. | 10
' -   : SV  GND_D ...... | ............. | Stromversorgung Digitalteil .. | 11
' -   : SV  P5V_A ...... | ............. | Stromversorgung Analogteil ... | 30
' -   : SV  GND_A ...... | ............. | Stromversorgung Analogteil ... | 31
' -   : SV  KON_A ...... | ............. | Referenzspannung Analogteil .. | 32
' -   : PRG /RESET ..... | ............. | Reset des MCs ................ |  9
' -   : QZ  Frequenz ... | ............. | Frequenznormal mit Quarz ..... | 12
' -   : QZ  Frequenz ... | ............. | Frequenznormal mit Quarz ..... | 13

' Port A    Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DE  T .......... | T_0 ......... | Tasteneingabe Taste 0 ........ | 40
' - .1: DE  T .......... | T_1 ......... | Tasteneingabe Taste 1 ........ | 39
' - .2: DE  T .......... | T_2 ......... | Tasteneingabe Taste 2 ........ | 38
' - .3: DE  T .......... | T_3 ......... | Tasteneingabe Taste 3 ........ | 37
' - .4: DE  T .......... | T_4 ......... | Tasteneingabe Taste 4 ........ | 36
' - .5: DE  T .......... | T_5 ......... | Tasteneingabe Taste 5 ........ | 35
' - .6: DE  T .......... | T_6 ......... | Tasteneingabe Taste 6 ........ | 34
' - .7: DE  T .......... | T_7 ......... | Tasteneingabe Taste 7 ........ | 33
'
'Port B     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: ................ | ............. | .............................. | ..
' - .1: ................ | ............. | .............................. | ..
' - .2: DE  WPM ........ | WPM ......... | Wecken aus Powerdown-Modus ... |  3
' - .3: DA  SE ......... | IRD ......... | Infrarot-Diode Modulation .... |  4
' - .4: ................ | ............. | .............................. | ..
' - .5: PRG MOSI ....... | PRG_MOSI .... | Programmierung ............... |  6
' - .6: PRG MISO ....... | PRG_MISO .... | Programmierung ............... |  7
' - .7: PRG SCK ........ | PRG_SCK ..... | Programmierung ............... |  8
'
'Port C     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DA  LCD ........ | LCD_E_KL .... | Anzeige (extern) ............. | 22
' - .1: DA  LCD ........ | LCD_RS_KL ... | Anzeige (extern) ............. | 23
' - .2: DA  LCD ........ | LCD_DB4_KL .. | Anzeige (extern) ............. | 24
' - .3: DA  LCD ........ | LCD_DB5_KL .. | Anzeige (extern) ............. | 25
' - .4: DA  LCD ........ | LCD_DB6_KL .. | Anzeige (extern) ............. | 26
' - .5: DA  LCD ........ | LCD_DB7_KL .. | Anzeige (extern) ............. | 27
' - .6: ................ | ............. | .............................. | ..
' - .7: ................ | ............. | .............................. | ..
'
'Port D     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DE RXD ......... | RXD_KL ...... | serielle Schnittstelle ....... | 14
' - .1: DA TXD ......... | TXD_KL ...... | serielle Schnittstelle ....... | 15
' - .2: DE IRE ......... | IRE_0 ....... | IR-Empfang Flanke fallend .... | 16
' - .3: DE IRE ......... | IRE_1 ....... | IR-Empfang Flanke steigend ... | 17
' - .4: ................ | ............. | .............................. | 18
' - .5: DA LED ......... | LED_1 ....... | Anzeige-LED 1 ................ | 19
' - .6: DA LED ......... | LED_2 ....... | Anzeige-LED 2 ................ | 20
' - .7: ................ | ............. | .............................. | 21
'
'12. Steckverbinder
'------------------
'X1 - Systemstecker
'                 SS
' SV  ==   ==   =======   =========================================+  LCD-Modul
'    ----------------------------------------------------------------
'   | 25 | 23 | 21 | 19 | 17 | 15 | 13 | 11 | 09 | 07 | 05 | 03 | 01 |
'    ----------------------------------------------------------------
'   | 26 | 24 | 22 | 20 | 18 | 16 | 14 | 12 | 10 | 08 | 06 | 04 | 02 |
'    ----------------------------------------------------------------
' SV  ==   ==   ==             ====================================+  Brenner
'
' 25, 26: GND    (Stromversorgung)
' 23, 24: P5 V   (Stromversorgung)
'                                                PRG-Pin
' 16    : GND    (Brenner, Stromversorgung) ........ 8
' 14    : P5 V   (Brenner, Stromversorgung) ........ 7
' 12    : /Reset (Brenner) ......................... 6
' 10    : MISO   (Brenner) ......................... 5
' 08    : MOSI   (Brenner) ......................... 4
' 06    : SCK    (Brenner) ......................... 3
' 04    : -      (Brenner, Verpolschutz) ........... 2
' 02    : -      (Brenner, Verpolschutz) ........... 1
'
'                                                LCD-Pin
' 17    : DB7    (LCD-Modul) ...................... 14
' 15    : DB6    (LCD-Modul) ...................... 13
' 13    : DB5    (LCD-Modul) ...................... 12
' 11    : DB4    (LCD-Modul) ...................... 11
' 09    : E      (LCD-Modul) ....................... 6
' 07    : -
' 05    : RS     (LCD-Modul) ....................... 4
' 03    : P5 V   (LCD-Modul, Stromversorgung) ...... 2
' 01    : GND    (LCD-Modul, Stromversorgung) ...... 1
'
'                                                 SS-Pin
' 19    : Rx     (Schnittstelle) ................... 3
' 20    : Tx     (Schnittstelle) ................... 2
' 21    : GND    (Schnittstelle) ................... 1
'
' Hinweis:
' Direkt am LCD-Module erfolgt
' - die Kontrasteinstellung ........................ 3
' - der Anschlu des Signales R/W auf Masse ........ 5
' - Hintergundbeleuchtung (Anode) ................. 15
' - Hintergundbeleuchtung (Kathode) ............... 16
'
' Die Anschlsse fr DB0-DB3 (Pin 7-10) bleiben unbeschaltet.
'
'-------------------------------------------------------------------------------
'
'Definition
'
   $regfile = "m16def.dat"                        'Prozessor ....... : ATmega16
   $crystal = 8000000                             'Quarzfrequenz ... : 8,0 MHz

'Compilereinstellung
'
   $hwstack = 96                                  'Hardware stack
   $swstack = 48                                  'Software stack
   $framesize = 48                                'Frame space
'
'-------------------------------------------------------------------------------
'
'Deklaration
'
'-Funktion
   Declare Function Tastatur()as Byte

'-Variable
   Dim Impuls As Word                             'Impuls  : Zeitdauer
   Dim Impe As Word                               'Impuls  : Zeitdauer ein
   Dim Impa As Word                               'Impuls  : Zeitdauer aus
   Dim Zeit As Word                               'Variable: Timerzeit
   Dim Z_pd As Long                               'Zhler: Powerdown   ( 3 S)
   Dim Z_pd2 As Long                              'Zhler: Powerdown 2 (10 M)

   Dim Fb As Byte                                 'Variable: Fernbedienung
   Dim Taste As Byte                              'Variable: Taste

'--Lnge der Impulse
   Dim L_min As Word                              'Impuls L .... : (Minimum)
   Dim L_max As Word                              'Impuls L .... : (Maximum)
   Dim H_min As Word                              'Impuls H .... : (Minimum)
   Dim H_max As Word                              'Impuls H .... : (Maximum)
   Dim S_min As Word                              'Synchronimpuls: (Minimum)
   Dim S_max As Word                              'Synchronimpuls: (Maximum)

'-Array
' Feldgre von 128 mu bleiben (4x4x8 Byte sind fr die Anzeige erforderlich)
   Dim A_ez(128) As Word                          'Array: empfangener Zeitwert
   Dim A_cp(16) As Byte                           'Array: Code des Protokolls
   Dim A_hx(4) As String * 16                     'Array: Hexzahl

'Array (Feldanzahl)
   Dim A_ezf As Integer                           'Feldanzahl: empfan. Zeitwert

'-Zhler
   Dim Hz01 As Byte                               'Zhler: Hilfszhler 01
   Dim Hz02 As Byte                               'Zhler: Hilfszhler 02
   Dim Hz03 As Byte                               'Zhler: Hilfszhler 03
   Dim Hz04 As Byte                               'Zhler: Hilfszhler 04

'-Flag
   Dim F_mrt As Bit                               'Flag: Modus (Rx/Tx)
   Dim F_mpd As Bit                               'Flag: Modus Powerdown
   Dim F_anz As Bit                               'Flag: Anzeige
   Dim F_lh As Bit                                'Flag: L- oder H-Impuls
   Dim F_bit As Bit                               'Flag: Statusbit (0/1)
   Dim F_start As Bit                             'Flag: Start "Empfang"

'-Merker
   Dim Hm01 As Byte                               'Merker: Hilfsmerker 01
   Dim Hm02 As Byte                               'Merker: Hilfsmerker 02
   Dim Hm03 As Byte                               'Merker: Hilfsmerker 03

   Dim M_fl As Byte                               'Merker: Anzahl der Flanken
   Dim M_tw As Byte                               'Merker: Tastaturwert
   Dim M_data As Byte                             'Merker: FB-Tastennummer
   Dim M_wid As Byte                              'Merker: Wiederholungen

   Dim M_lcd As String * 10                       'Merker: Anzeige

'-String
   Dim Hs01 As String * 1                         'String: Hilfssstring 01
   Dim Hs02 As String * 1                         'String: Hilfssstring 02
'
'-------------------------------------------------------------------------------
'
'I/O-Ports Konfiguration
'
'=Port A
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddra = &B0000_0000                             'PB0-7 Ein
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Porta = &B1111_1111                            'PB0-7 aktiv

'=Port B
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrb = &B0000_1100                             'PB0-1, 4-7 Ein; PB2-3 Aus
                                                  'INT 2 an PB2 als Output!

'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portb = &B0000_0100                            'PB0-1, 3-7 passiv; 2 aktiv

'=Port C
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrc = &B1111_1111                             'PB0-7 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portc = &B0000_0000                            'PB0-7 passiv

'=Port D
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrd = &B0110_0001                             'PD1-4,7 Ein; PD0,5-6 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portd = &B1010_1100                            'PD2-3,5,7 akt., PD0-1,4,6 pa.
'
'-------------------------------------------------------------------------------
'
'Timer Konfiguration
'-Timer0: CTC-Konfiguration (Modulationsfrequenz)
'-Zeitbasis: Quarz/ 2/ Teiler = Frequenz ( 8 MHz/ 2/ 110= 36 kHz, T= 27,5 s)
'Frequenzen bei 8 MHz-Quarz
' OCR0= 125: 31,9 kHz   OCR0= 120: 33,2 kHz   OCR0= 115: 34,5 kHz
' OCR0= 110: 36,2 kHz   OCR0= 105: 37,9 kHz
'
    Ocr0 = 110                                    'Frequenzeinstellung
    Portb.3 = 1                                   'Sende-Ende: IR-Diode aus

'-Timer1: Normal-Konfiguration (Zeitbasis fr Kodierung; 1-8-64-256-1024)
'-Zeitbasis: T = 1/ Quarz* Teiler (1/  8,0 MHz* 8 = 1,0 s)
'
   Config Timer1 = Timer , Prescale = 8           'Konfiguration

'-Timer2: Normal-Konfiguration (Zeitbasis fr Powerdown; 1-8-64-256-1024)
'         Das Aufwecken erfolgt durch einen Interrupt 2 (Pin 3)
'-Zeitbasis: T = 1/ Quarz* Teiler (1/  8,0 MHz* 1024* 256= 32,768 ms)
'         Der Timerinterrupt 2 wird beim Timerberlauf ausgelst.
'
   Config Timer2 = Timer , Prescale = 1024        'Konfiguration
'
'-------------------------------------------------------------------------------
'
'LCD Konfiguration
'-LCD-Typ ..... : 2 Zeilen, 16 Zeichen
'-Datenport ... : Port C zur Datenbertragung erfordert fr das JTAG(EN)-Flag:
'                 Einstellung "disable" (FUSE-Bit)
'
   Config Lcd = 16 * 2
   Config Lcdpin = Pin , Db4 = Portc.2 , Db5 = Portc.3 , Db6 = Portc.4 , _
          Db7 = Portc.5 , E = Portc.0 , Rs = Portc.1
'
'-------------------------------------------------------------------------------
'
'Konfiguration Interrupt
'
   Config Int0 = Falling                          'Signal: fallende Flanke
   Config Int1 = Rising                           'Signal: steigende Flanke
   Config Int2 = Falling                          'Signal: fallende Flanke

   Enable Int0                                    'Einschalten: Interrupt INT0
      On Int0 Int_int0                            'Interrupt 0: fallende Flanke
   Enable Int1                                    'Einschalten: Interrupt INT1
      On Int1 Int_int1                            'Interrupt 1: steigende Flanke
   Enable Int2                                    'Einschalten: Interrupt INT2
      On Int2 Int_int2                            'Interrupt 2: steigende Flanke

   Disable Timer1                                 'Interrupt: Timer 1
      On Timer1 Int_tim1                          'Timer 1: berlauf

   Enable Timer2                                  'Interrupt: Timer 2
      On Timer2 Int_tim2                          'Timer 2: berlauf
   Enable Interrupts                              'Interrupts : global zulassen
'
'-------------------------------------------------------------------------------
'
'Initialisierung
'
   Cursor Off                                     'LCD-Cursor ausschalten
   A_ezf = 128                                    'Feldanzahl: Array Zeitwerte
   Portd.5 = 0
   F_mpd = 1 : F_mrt = 0                          'Start nach dem Einschalten
'                                                  bzw. Programmieren
'-------------------------------------------------------------------------------
'
'Programm
'Hinweis: Alle Befehle "LCD" beziehen sich auf die Ausgabe auf einem LCD-Modul,
'         welches entsprechend der Steckerbelegung angeschlossen sein mu.
Do
M_do:
   Gosub Up_powerdown                             'UP: Powerdown
'Kontrolleintrag
'   Cls : Lcd "AVR lebt" : Wait 3                  '+++++ auskommentiert

   Select Case F_mrt                              'Modus abfragen (Rx/Tx)
      Case 0                                      'Modus: Tx (Senden)
'Kontrolleintrag
'   Cls : Lcd "Makro senden" : Wait 3              '+++++ auskommentiert
         Gosub Up_makro
'
'===============================================================================
'
      Case 1                                      'Modus: Rx (Empfangen)
'Kontrolleintrag
'   Cls : Lcd " 1. Empfang" : Wait 3               '+++++ auskommentiert
         Disable Int2                             'Interrupt 2 sperren
M_c1:
         Gosub Up_empfangen
'Abfrage, ob neuen Befehl lesen
         Cls
         Lcd "1: neues Lesen" : Locate 2 , 1 : Lcd "2: ESC"
         Taste = Tastatur()
'Kontrolleintrag
'         Cls : Lcd Taste                          '+++++ auskommentiert
'         Locate 2 , 1 : Lcd "bettigt" : Wait 3   '+++++ auskommentiert

         If Taste <> 1 Then
            F_mrt = 0                             'Modus: Tx (Senden)
         Else
            Goto M_c1
         End If
      Enable Int2                                 'Interrupt 2 freigeben
   End Select

M_ende:
Loop
'
'-------------------------------------------------------------------------------
'=== U N T E R P R O G R A M M =================================================
'-------------------------------------------------------------------------------
'
Up_makro:
'Unterprogramm: Makrowahl
'
M_nw:
'Kontrolleintrag
'   Cls : Lcd " + Makrowahl +"                     '+++++ auskommentiert

'Impulsanfang
' Taste anzeigen
'Kontrolleintrag
'   Locate 2 , 1 : Lcd "Makro " + Str(taste)       '+++++ auskommentiert
'   Wait 3                                         '+++++ auskommentiert

'LED zur Bettigung der Befehlseingabe kurz leuchten lassen
   If Taste < 255 Then
      Portd.5 = 1 : Waitms 100 : Portd.5 = 0 : Waitms 100
   End If

'Makro entsprechende der Taste
   Select Case Taste
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Makro 1 [AIWA]: 1x "program 1"
'Makro 2 [AIWA]: 8x "sleep"
'==============================
      Case 1 To 2                                 'Makro: Taste 1, 2
'Gert festlegen
         Fb = 1                                   'Fernbedienung 1: AIWA
'Impulslngen festlegen
         Gosub Up_il
'Mittelwerte der Zeiten fr "Ein" und "Aus" berechnen (Shift: /2 teilen)
         L_min = L_min + L_max : Shift L_min , Right
         H_min = H_min + H_max : Shift H_min , Right
         S_min = S_min + S_max : Shift S_min , Right
'Zusammenstellung der Tastenkombination
         Select Case Taste
            Case 1                                'Makro 1
               M_data = 1                         'AIWA-Taste "1"
               M_wid = 1                          'Senden: 1x
               Gosub Up_am01
            Case 2                                'Makro 2
               M_data = 2                         'AIWA-Taste "sleep"
               M_wid = 8                          'Senden: 8x
               Gosub Up_am01
         End Select
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Makro 3
'=======
'      Case 3
'         ...
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Case Else
'Kontrolleintrag
'         Cls : Lcd "Kein Makro!"                  '+++++ auskommentiert
'         Wait 1                                   '+++++ auskommentiert
         Goto M_ende
      End Select
      Taste = 255                                 'Auswahl lchen
M_e:

Return
'
'-------------------------------------------------------------------------------
'
Up_am01:
'Unterprogramm: AIWA-Makro 01
' AIWA-Makro fr die Taste, die in M_data festgelegt ist
'
'Schleifensteuerung zur Tastenwiederholung
   While M_wid > 0                                'Schleife: Wiederholung
'Sendebetrieb
      F_lh = 0                                    'Zeitinterval mit Modulation
      Hm01 = Lookup(fb , G_sya)                   'FB-Impulsanzahl ermitteln

      For Hz02 = 1 To Hm01                        'Schleife: Startimpulse
         Select Case Hz02
            Case 1
               Impuls = Lookup(fb , G_sya1)       '1. Start-Impuls lesen
            Case 2
               Impuls = Lookup(fb , G_sya2)       '2. Start-Impuls lesen
            Case 3
               Impuls = Lookup(fb , G_sya3)       '3. Start-Impuls lesen
            Case 4
               Impuls = Lookup(fb , G_sya4)       '4. Start-Impuls lesen
            Case 5
               Impuls = Lookup(fb , G_sya5)       '5. Start-Impuls lesen
         End Select
         Gosub Up_senden                          'Start-Impuls senden
      Next

      Select Case M_data                          'DATA-Feld je nach Funktion
         Case 1                                   '1: Taste "program 1"
            Hm01 = Lookup(0 , Aiw_1)              'Ermittlung der Codelnge
            Hm03 = Hm01 + 1
            Hm03 = Lookup(hm03 , Aiw_1)           'Bit-Anzahl am Code-Ende

         Case 2                                   '2: Taste "sleep"
            Hm01 = Lookup(0 , Aiw_sleep)          'Ermittlung der Codelnge
            Hm03 = Hm01 + 1
            Hm03 = Lookup(hm03 , Aiw_sleep)       'Bit-Anzahl am Code-Ende

         Case 3                                   '3: Taste "xxx"
'           ......

      End Select

'Sequenzen auslesen und senden
      Decr Hm03                                   '-1, da Zhlung von 0 bis n
      For Hz01 = 1 To Hm01                        'Schleife: Sende-Sequenz
         Select Case M_data                       ' DATA-Feld je nach Funktion
            Case 1                                '1: Taste "1"
               Hm02 = Lookup(hz01 , Aiw_1)        '   Byte von Data lesen
            Case 2                                '2: Taste "sleep"
               Hm02 = Lookup(hz01 , Aiw_sleep)    '   Byte von Data lesen
            Case 3
'              ......

         End Select

         For Hz02 = 0 To 7                        'Sequenz durchlaufen (Byte)
'Abbruch beim letzten Byte, wenn Bits fehlen (Hm03: Bitanzahl im letzten Byte)
            If Hz01 = Hm01 And Hz02 > Hm03 Then
               Exit For
            End If
'Synchron-Impuls senden
            Impuls = S_min
            Gosub Up_senden
'Daten-Impuls ermitteln und senden
            F_bit = Hm02.hz02                     'Bit: 0/1 ermitteln
            If F_bit = 0 Then
               Impuls = L_min
            Else
               Impuls = H_min
            End If
            Gosub Up_senden                       'Impuls senden
         Next
      Next

'Ende-Sequenz senden
      Hm01 = Lookup(fb , G_sye)                   'Impulsanzahl ermitteln
      For Hz02 = 1 To Hm01
         Select Case Hz02
            Case 1
               Impuls = Lookup(fb , G_sye1)       '1. Ende-Impuls lesen
            Case 2
               Impuls = Lookup(fb , G_sye2)       '2. Ende-Impuls lesen
            Case 3
               Impuls = Lookup(fb , G_sye3)       '3. Ende-Impuls lesen
            Case 4
               Impuls = Lookup(fb , G_sye4)       '4. Ende-Impuls lesen
            Case 5
               Impuls = Lookup(fb , G_sye5)       '5. Ende-Impuls lesen
         End Select
         Gosub Up_senden                          'Ende-Impuls senden
      Next

      Waitms 10                                   'Pause bei den Wiederholungen
      Decr M_wid                                  'Schleifenzhler reduzieren
   Wend                                           'Ende der Schleife

Return
'
'-------------------------------------------------------------------------------
'
Up_senden:
'Unterprogramm: Senden
'Schleifensteuerung fr die IR-Diode
' F_lh = 0 [1] : Zeitinterval mit [ohne] Modulation
'
   Impuls = Impuls * 5                            '5: Faktor bei 8,0 MHz
   Select Case F_lh
       Case 0                                     'mit Modulation (1. Aufruf)
'mit Modulation
'Schleife, wenn Periode noch aktiv
'Frequenz einschalten (Timer0)
         Tccr0 = &B0001_1001                      'Timer0: CTC-Modus
         Timer1 = 0
         While Timer1 < Impuls                    'Zeitschleife bei "0"
         Wend
'Frequenz ausschalten (Timer0)
         Tccr0 = &B0000_1001                      'OC0-Ausgang von PB.3 trennen
         Portb.3 = 1                              'Sende-Ende: IR-Diode aus
      Case 1
'ohne Modulation
'Schleife, wenn Periode noch aktiv
         Timer1 = 0
         While Timer1 < Impuls                    'Zeitschleife bei "1"
         Wend
   End Select
   Toggle F_lh                                    'Flanke wechseln (0>1 / 1>0)

Return
'
'-------------------------------------------------------------------------------
'
Up_empfangen:
'Unterprogramm: Empfangen
'- Befehl der Fernbedienung lesen
'- Werte auf der LCD-Anzeige ausgeben
'
'Array auf der LCD-Anzeige ausgeben
   For Hz01 = 1 To A_ezf                          'Array lschen ()
      A_ez(hz01) = 0
   Next Hz01
   Hz01 = 1                                       'Array: Zhler rcksetzen
   Cls : Lcd "FB waehlen"
   Wait 1
   Fb = 1
Mfb1:
      Portd.6 = 0                                 'Anzeige: "Code empfangen" aus
      Cls : Lcd Lookupstr(1 , Gt)                 'Anzeige: Gertetyp (1: AIWA)
Mfb2:
      Locate 2 , 1 : Lcd "1:>  2:ESC  3:OK"
      Taste = Tastatur()                          'Tastaturabfrage
      Select Case Taste
         Case 1                                   'Tastatur: Gert whlen
            Incr Fb
            If Fb > 3 Then Fb = 1                 'Gerteanzahl [3]
            Locate 1 , 1 : Lcd Lookupstr(fb , Gt) 'Anzeige: Gert
         Case 2
            F_mrt = 0
            Goto M_do                             'Abbruch der Routine
         Case 3
            Goto Mfb3                             'Tastatur: Auswahl getroffen
      End Select
      Goto Mfb2
Mfb3:
   Cls : Lcd "-Empfangsbereit-"
   Locate 2 , 1 : Lcd "  FB bedienen"

'Einlesen des Codes
   Enable Timer1                                  'INT: berlauf Timer1 ein
   F_start = 1
Mfb4:
   Taste = Tastatur()                             'Tastaturabfrage
   Disable Timer1                                 'INT: berlauf Timer1 aus
Mfb5:
   Select Case Taste
      Case 1                                      'Taste 1: Nachfolger ansehen
         Gosub Up_anzeige
         If Hz01 > 2 And A_ez(hz01 -1) = 0 Then
            Goto Mfb6
         End If
      Case 2                                      'Taste 2: Vorgnger ansehen
         If Hz01 > 2 Then
            Hz01 = Hz01 - 2
         Else
            Hz01 = A_ezf - 1
            While A_ez(hz01) = 0                  'letztes Element ermitteln
               Decr Hz01
            Wend
         End If
         Gosub Up_anzeige
      Case 3                                      'Taste 3: Anzeige HEX-Code
         Gosub Up_code                            'Anzeige des Codes
         F_anz = 1
         If Taste = 1 Then
            Goto Mfb5                             'Anzeige Wiederholung
         Elseif Taste = 2 Then
            Goto Mfb6                             'Anzeige Ende
         End If
   End Select
   Goto Mfb4
Mfb6:

Return
'
'-------------------------------------------------------------------------------
'
Up_code:
'Unterprogramm: HEX-Wandlung
'-Die eingelesenen Zeiten werden entsprechend der Gerteauswahl in eine hexa-
' dezimale Zahl umgewandelt.
'
'-Ermittlung, ab welcher Zeitsequenz der Code gebildet werden soll
' Dieser Wert ist um 1 grer, als der Wert, der die Anzahl der Synchronimpulse
' am Anfang angibt.
'
   Gosub Up_il
   H_min = H_min * 5                              '5: Faktor bei 8,0 MHz
   H_max = H_max * 5                              '5: Faktor bei 8,0 MHz

   Hz04 = Lookup(fb , G_is)                       'Synchonimpuls ja/nein
   Incr Hm01                                      '1. Zeitsequenz fr den Code
   If Hz04 = 1 Then Incr Hm01

   Hz01 = Hm01                                    'Zhler: Feld-Zeitprotokoll
   Hz03 = 1                                       'Zhler: Code des Protokolls

Mc00:
   For Hz02 = 0 To 7
      If A_ez(hz01) > H_max Then                  'Sequenz-Ende erreicht
         Goto Mh00
      End If

      If A_ez(hz01) > H_min And A_ez(hz01) < H_max Then
         Set A_cp(hz03).hz02                      'H_min < x < H_max: => 1
      Else
         Reset A_cp(hz03).hz02                    'L_min < x < L_max: => 0
      End If
Mc01:
      Incr Hz01                                   'nchsten Impuls auswerten
      If Hz04 = 1 Then Incr Hz01                  'Synchronimpuls ausblenden
      If Hz01 = A_ezf Then Goto Mh00              'Abbruch bei Feldende
   Next

   Incr Hz03                                      'nchtes Codebyte schreiben
   Goto Mc00                                      'Sprung an den Anfang

'Anzeigetext generieren
Mh00:
   Hz02 = 1                                       'Zhler fr das Textarray
While Hz02 < 5                                    'Schleife 4x durchlaufen
   A_hx(hz02) = Str(hz02) + ": "
   For Hz01 = 1 To 4
      Hz03 = Hz02 - 1
      Hz03 = Hz03 * 4
      Hz03 = Hz03 + Hz01

      Hm01 = &B0000_1111 And A_cp(hz03)           'niederwertiger Code-Teil
      Hs01 = Lookupstr(hm01 , Bth)
      Hm02 = &B1111_0000 And A_cp(hz03)           'hherwertiger Code-Teil
      Shift Hm02 , Right , 4
      Hs02 = Lookupstr(hm02 , Bth)

      Hs01 = Hs02 + Hs01
      A_hx(hz02) = A_hx(hz02) + Hs01
      If Hz01 < 4 Then
         A_hx(hz02) = A_hx(hz02) + "-"            'Anzeigetext formatieren
      End If
   Next
   Incr Hz02
Wend

'Code anzeigen
   For Hz01 = 1 To 3 Step 2
   Cls : Lcd A_hx(hz01)
   Hz02 = Hz01 + 1
   Locate 2 , 1 : Lcd A_hx(hz02)
Mh01:
      Taste = Tastatur()
   Next

Return
'
'-------------------------------------------------------------------------------
'
Up_il:
'Unterprogramm: Impulslnge
'- Anzahl und Lnge der Impulse
'  (FB ist die Gertenummer 1 ... x)
'
   Hm01 = Lookup(fb , G_sya)                      'Anzahl der Synchronimpulse

   S_min = Lookup(fb , G_is1)                     'Synchronimpuls: Minimum
   S_max = Lookup(fb , G_is2)                     'Synchronimpuls: Maximum

   L_min = Lookup(fb , G_l1)                      'Zeitsequenz L: Minimum
   L_max = Lookup(fb , G_l2)                      'Zeitsequenz L: Maximum

   H_min = Lookup(fb , G_h1)                      'Zeitsequenz H: Minimum
   H_max = Lookup(fb , G_h2)                      'Zeitsequenz H: Maximum

Return
'
'-------------------------------------------------------------------------------
'
Up_anzeige:
'Unterprogramm: Anzeige
'- Befehl der Fernbedienung lesen
'
   If F_anz = 1 Then                              'Durchlauf nur 1x
      F_anz = 0
      Hz01 = 1
   End If

   M_lcd = Str(hz01)

S_lcd:
   If Len(m_lcd) < 3 Then                         'Code-Nummer: 3 stellig
      M_lcd = "0" + M_lcd
      Goto S_lcd
   End If

   M_lcd = M_lcd + ": " + Str(a_ez(hz01))
   Cls                                            'Anzeige: lschen
   Lcd M_lcd                                      'Anzeige: Zeitwert anzeigen
   Locate 2 , 1 : Lcd "1:>  2:<  3:Code"
   If Hz01 < 200 Then Incr Hz01                   'Array: hochzhlen

Return
'
'-------------------------------------------------------------------------------
'
Up_powerdown:
'Unterprogramm: AVR in Powerdown
' AVR in den Modus "Powerdown" versetzen
   If F_mpd = 1 Then                              'Zeit erreicht und Modus Tx
      F_mpd = 0
      Cls : Lcd "Powerdown"
      Enable Int2
      Powerdown
      Z_pd = 0                                    'Reset Powerdown   ( 3 S)
      Z_pd2 = 0                                   'Reset Powerdown 2 (10 M)
   End If

Return
'
'-------------------------------------------------------------------------------
'=== F U N K T I O N ===========================================================
'-------------------------------------------------------------------------------
'
'Funktion: Tastenabfrage
'- erst bei Bettigungsende Rckkehr
'
Function Tastatur:

Me01:
   Gosub Up_powerdown                             'UP: Powerdown
   Taste = Pina                                   'Portabfrage
   Z_pd = 0                                       'Powerdown unterdrcken
   If Taste = 255 Then Goto Me01                  'keine Taste (Rcksprung)
Mt01:
   Z_pd = 0                                       'Powerdown unterdrcken
   Waitms 20
   If Taste = Pina Then Goto Mt01
   M_tw = Taste
   Taste = Lookdown(m_tw , Ttc , 10)              'Ttc: Tabelle Tastencode
   Taste = Taste - 1
'Reset fr Powerdown bei Tastenbettigung
   Z_pd2 = 0

'Kontrolleintrag
'   Cls : Lcd Taste                                '+++++ auskommentiert
'   Locate 2 , 1 : Lcd "T-Abfrage" : Wait 5        '+++++ auskommentiert

End Function
'
'-------------------------------------------------------------------------------
'=== I N T E R R U P T R O U T I N E ===========================================
'-------------------------------------------------------------------------------
'
Int_tim1:
'Interrupt: Timer 1
'- berschreiten des Zeitintervalls beim Lesen des Befehlscodes der FB
'Anzeige: Aufnahmeende
'Blinken, wenn berlauf Timer 1 (Aufnahmeende)
'
      If A_ez(1) <> 0 And A_ez(a_ezf) = 0 Then    'letztes Feldelement
         A_ez(a_ezf) = 1                          'Merken: Aufnahmeende erreicht
'Blinken
         Waitus 50
         For Hz01 = 1 To 6                        'LED blinkt
            Toggle Portd.5 : Waitms 200
         Next

         Cls : Lcd "Code"
         Locate 2 , 1 : Lcd "1: anzeigen"
      End If
M_t1:

Return
'
'-------------------------------------------------------------------------------
'
Int_tim2:
'Interrupt: Timer 2
'-nach einer Zeit wird in den Modus "Powerdown" geschaltet
   If F_mrt = 0 Then
      Incr Z_pd                                   'Zhler: berlauf Timer 2
   Else
      Incr Z_pd2                                  'Zhler: berlauf Timer 2
   End If

' Zeitbasis: 32,768 ms => 3 s: 92
   If Z_pd > 92 Then                              'AVR abschalten
      F_mpd = 1                                   'Modus: Powerdown
   End If
'-nach Zeit 2 (> Zeit 1) wird die Fernbedienung ausgeschaltet, wenn Sie sich im
' Empfangsmodus befindet
' Zeitbasis: 32,768 ms => 10 Minuten:
   If Z_pd2 > 18310 Then                          'AVR abschalten
      F_mpd = 1                                   'Modus: Powerdown 2
   End If

Return
'
'-------------------------------------------------------------------------------
'
Int_int0:
'Interrupt 0: Signalflanke fallend
'- Befehlscode der Fernbedienung fr einen Befehl lesen
'
   If F_start = 1 Then                            'Empfang freigegeben
      Timer1 = 0
      F_start = 0
      F_anz = 1                                   'Flag: Anzeige
      Goto Mi0                                    '1. Interrupt wird ignoriert
   End If

   A_ez(hz01) = Timer1                            'Zeit in Array (x) schreiben
   Timer1 = 0                                     'Timer1: Reset
   Incr Hz01                                      'x = x + 1
Mi0:

Return
'
'-------------------------------------------------------------------------------
'
Int_int1:
'Interrupt 1: Signalflanke steigend
'- Befehlscode der Fernbedienung fr einen Befehl lesen
'
   A_ez(hz01) = Timer1                            'Zeit in Array (x+1) schreiben
   Timer1 = 0                                     'Timer1: Reset
   Incr Hz01                                      'x = x + 1
Mi1:

Return
'
'-------------------------------------------------------------------------------
'
Int_int2:
'Interrupt 2: Signalflanke fallend
'- AVR wird wieder aktiviert
'
'Wenn Taste 1 und 8 zusammen gedrckt ist, dann in Modus "Empfangen" schalten.
'Dazu mu sichergestellt sein, da die Brcke zur Stromversorgung des Empfnger-
' ICs eingesetzt ist.

'Jede Tastenbedienung setzt die Zhler fr die Powerdown-Modi zurck
   Cls
   Z_pd = 0                                       'Reset fr Powerdown
   Z_pd2 = 0                                      'Reset fr Powerdown 2
   Waitms 50                                      'Unterschiede beim Tasten-
                                                  ' drcken ausschalten (1&8)
   Taste = Pina                                   'Portabfrage
   If Taste = 255 Then Goto M_int2                'bei Taste 255
'Nummer der Taste ermitteln
   Taste = Lookdown(taste , Ttc , 10)             'Ttc: Tabelle Tastencode
   Taste = Taste - 1
'Kontrolleintrag
'   Lcd Taste : Wait 3                             '+++++ auskommentiert

   If Taste <> 9 Then                             'Taste 1&8 bettigt
      F_mrt = 0                                   'Modus: Senden
'Kontrolleintrag
'      Cls : Lcd " Modus: Tx "                     '+++++ auskommentiert
'      Wait 1                                      '+++++ auskommentiert
   Else
      Toggle F_mrt                                'Modus: Empfangen Ein/Aus
'Begrung
      If F_mrt = 0 Then Goto M_int2               'wenn Aus, dann beenden
      Cls : Lcd " Fernbedienung"
      Locate 2 , 1 : Lcd "   AMATRONIK"
      Wait 2
      Locate 2 , 1 : Lcd " Modus: Rx  "           'Modus: Empfangen
      Wait 3
   End If

M_int2:
Return

'Programmende
End
'
'===============================================================================
'
'Datenbereich
'-Code fr Tastaturabfrage
Ttc:                                              'Tabelle: TastenCode
   Data 255 , 254 , 253 , 251 , 247 , 239 , 223 , 191 , 127 , 126
'Taste:   % -   1 -   2 -   3 -   4 -   5 -   6 -   7 -   8 - 1&8
'
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'-Binr zu Hex
Bth:
   Data "0" , "1" , "2" , "3" , "4" , "5" , "6" , "7" , "8" , "9" , "A" , "B"
   Data "C" , "D" , "E" , "F"
'
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'-Gertetypen
Gt:
   Data "" , "Audio - AIWA" , "TV - SEG    " , "SAT - Strong"
'
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'-Konstante Gert
' Die Zeitabtastung erfolgt mit Zeitschritten von 2 s.
' => Wert x 2 s = Zeitdauer
' Die im Array stehenden Werte mssen vorher mit 100 multipliziert werden.
' Die Daten werden entsprechend den Gerten im DATA-Feld abgelegt.
' Reihenmfolge:
' 1. Audio - AIWA, 2. TV - SEG, 3. SAT - Strong
'
' Je nach verwendeter Quarzfrequenz mssen die eingetragenen Werte mit einem
' Faktor multipiziert werden:
'  8,0 MHz: Faktor  5
' 16,0 MHz: Faktor 10

'Synchronimpulse
G_sya:
'Byteabfrage, => kein % (Integer-Typ)             'Synchronimpulse am Anfang
   Data 0 , 2                                     'der Sequenz

G_sya1:                                           'Lnge des 1. Synchronimpulses
   Data 0% , 1810%                                'mit Faktor multiplizieren

G_sya2:                                           'Lnge des 2. Synchronimpulses
   Data 0% , 890%                                 'mit Faktor multiplizieren

G_sya3:                                           'Lnge des 3. Synchronimpulses
   Data 0% , 0%                                   'mit Faktor multiplizieren

G_sya4:                                           'Lnge des 4. Synchronimpulses
   Data 0% , 0%                                   'mit Faktor multiplizieren

G_sya5:                                           'Lnge des 5. Synchronimpulses
   Data 0% , 0%                                   'mit Faktor multiplizieren

G_sya6:                                           'Lnge des 6. Synchronimpulses
   Data 0% , 0%                                   'mit Faktor multiplizieren

G_sya7:                                           'Lnge des 7. Synchronimpulses
   Data 0% , 0%                                   'mit Faktor multiplizieren

G_sya8:                                           'Lnge des 8. Synchronimpulses
   Data 0% , 0%                                   'mit Faktor multiplizieren

'Synchronisationimpuls vor dem Code
G_is:                                             'Impulssynchronisation
'Byteabfrage, => kein % (Integer-Typ)
   Data 0 , 1                                     ' nein: 0   ja: 1

G_is1:                                            'Lnge der Impulssynchronis.
   Data 0% , 110%                                 ' mit 10 multiplizieren

G_is2:                                            'Lnge der Impulssynchronis.
   Data 0% , 120%                                 ' mit 10 multiplizieren

'Impulslngen fr den Code
G_l1:                                             'Zeitdauer L (Aus): Minimum
   Data 0% , 130%                                 ' mit 10 multiplizieren

G_l2:                                             'Zeitdauer L (Aus): Maximum
   Data 0% , 140%                                 ' mit 10 multiplizieren

G_h1:                                             'Zeitdauer H (Ein): Minimum
   Data 0% , 310%                                 ' mit 10 multiplizieren

G_h2:                                             'Zeitdauer H (Ein): Maximum
   Data 0% , 370%                                 ' mit 10 multiplizieren

'Ende der Impulsbertragung
G_sye:                                            'Anzahl der Synchronimpulse am
'Byteabfrage, => kein % (Integer-Typ)              Ende der Sequenz
   Data 0 , 5

G_sye1:                                           'Lnge der Impulssynchronis.
   Data 0% , 125%                                 ' mit 10 multiplizieren

G_sye2:                                           'Zeitdauer
   Data 0% , 4600%                                ' mit 10 multiplizieren

G_sye3:                                           'Zeitdauer
   Data 0% , 1820%                                ' mit 10 multiplizieren

G_sye4:                                           'Lnge der Impulssynchronis.
   Data 0% , 880%                                 ' mit 10 multiplizieren

G_sye5:                                           'Zeitdauer
   Data 0% , 125%                                 ' mit 10 multiplizieren
'
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'Sendecode fr die Fernbedienungen
' Das erste Byte beinhaltet die Lnge des Sendecodes, das letze die Anzahl der
' nicht zu sendenden Bits.
'- Audio-AIWA
'   Taste 1
Aiw_1:                                            '5 Byte, ..., 2 Bits Code-Ende
   Data &H06 , &H79 , &HC0 , &HF0 , &H07 , &HF8 , &H03 , &H02

Aiw_2:                                            '5 Byte, ..., 2 Bits Code-Ende
   Data &H06 , &H79 , &HC0 , &HF0 , &H0B , &HF4 , &H03 , &H02

Aiw_sleep:                                        '5 Byte, ..., 2 Bits Code-Ende
   Data &H06 , &H79 , &HC0 , &HF0 , &H17 , &HE9 , &H02 , &H02

'Programmende